/* Copyright 1998 Sam Trenholme
   This code is released to the public domain. */

/* Input filter for incoming mail, determines if the message
   is within the window of allowed time stamps */

#include <time.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <fcntl.h>
#include "kiwi.h"

char b32map[] = MAPPING;

int
main()
{

time_t time;
char input[8], *formatstring;
message msg[1];

int counter, timedata=0, ipdata=0;

/* Get the message we wish to decode from the standard input */
for(counter=0;counter<7;counter++)
  {
  input[counter] = getc(stdin);
  }

input[7]=0;

/* Convert the ASCII-encoded encrypted message in to a simple structure in 
   the format {data format, data} */
parsedate(input,msg);

/* And, depending on the type of format the data is in, set the string
   "formatstring" to a description of the data format */
switch(msg->format)
	{
	case FORMAT_INVALID: 
		formatstring = "Invalid data."; 
		break;
	case FORMAT_SHORT_TIMEOUT: 
		formatstring = "Short timeout"; 
		timedata=1;
		break;
	case FORMAT_MID_TIMEOUT: 
		formatstring = "Mid timeout"; 
		timedata=1;
		break;
	case FORMAT_LONG_TIMEOUT: 
		formatstring = "Long timeout"; 
		timedata=1;
		break;
	case FORMAT_SUBJECT_HASH: 
		formatstring = "Subject hash"; 
		break;
	case FORMAT_IP_BLOCK: 
		formatstring = "Truncated IP"; 
		ipdata=1;
		break;
	case FORMAT_HIDDEN_DIR: 
		formatstring = "Moving directory"; 
		timedata=1;
		break;
	case FORMAT_FIVE_LETTERS:
		formatstring = "5-letter embedded code";
		break;
	case FORMAT_BAD_PARITY: 
		formatstring = "Bad parity--no data"; 
		break;
	}

/* Tell them the data format */
printf("Data type: %s\n",formatstring);

/* If the message is a timestamp, tell them what the time stamp is in 
   both UNIX and English time */
if(timedata) 
	{
	printf("Message: %d, or %s",msg->data,ctime((time_t *)&msg->data)); 
	}

/* If the message is an IP block, tell them what IP block it is */
if(ipdata)
	{
	u_int32_t ip;
	u_int8_t ipo[4];
	ip = msg->data;
	for(counter=3;counter>=0;counter--)
		{
		ipo[counter] = ip & 0xff;
		ip >>= 8;
		}
	printf("Message: %d.%d.%d.%d-%d\n",ipo[0],ipo[1],ipo[2],
               ipo[3],ipo[3]+16);
	}

/* If the message is a five-letter embedded message (for mailing lists and 
   other permanent email addresses), tell them what the five letters are */
if(msg->format == FORMAT_FIVE_LETTERS)
	{
	char mesg[6];
	for(counter=4;counter>=0;counter--)
		{
		mesg[counter] = b32map[msg->data & 31];
		msg->data >>= 5;
		}

	mesg[5] = 0;

	printf("Message: %s\n",mesg);

	}

}

