/* Copyright 1998, 1999 Sam Trenholme
   This code is released to the public domain. */

/* Input filter for incoming mail, determines if the message
   is within the window of allowed time stamps */

#include <time.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <fcntl.h>
#include "kiwi.h"

/* 

   This program takes from the standard input an email message and
   outputs to standard output the email address the message in question 
   is delivered to.

   This program assumes a qmail-system with a "Delivered-to" header.

   In "mail envelopes" with multiple "Delivered-to" headers, it assumes
   that the last delivered-to header is the canonical one.

*/

/* Given a line in the form From: someone <name@some.domain.com> 
   give us the email address in the string answer, e.g.
   name@some.domain.com */

char *getaddress(char *line, char *answer)
{

char *startword;
char *point;

int counter=0, inspace=1, isaddr=0, max=BUFMAX;
if(line==NULL || line==0)
	{
	if(answer != NULL && answer != 0)
		*answer = 0;		
  	return NULL;
	}

/* This needs to be more RFC822-complient */

point = startword = line;

/* This is very simple.  All it does is look for a word separated either by
   whitespace or '<' or '>', contining an '@', and assume that the word
   in question is an email address */

while(*point != 0 && counter < BUFMAX)
	{
	if(inspace && *point > 32 && *point != '<' && *point != '>')
		{
		inspace = 0;
		startword = point;
		}
	if(*point <= 32 || *point == '<' || *point == '>') 
		inspace = 1;
	if(*point == '@')
		break;
	counter++;
	point++;
	}

counter = 0;
max = BUFMAX;
max -= (startword - line);

/* If we found an email address above, copy the email address to the 
   string 'answer' */

if(startword != line)
	{
	while(startword[counter] > 32 && startword[counter] != '>'
	      && startword[counter] != '<' && counter < max)
		{
		answer[counter] = startword[counter];
		counter++;
		}
	}

answer[counter] = 0;
		
return answer;

}

/* Tell us whether the email address in a given line, such as
   for kiwi-bhqgjaf@koala.samiam.org, decrypts to valid data, and,
   if so, whether the data gives the person permission to email us */

goodtime(char *line) 
{ 

char date[BUFMAX];
char dline[BUFMAX];
char *lastdash, *index;
int counter;
time_t now,mailtime;

message msg[1];

*dline = 0;
lastdash = index = dline;

getaddress(line,dline);

/* Look for the last 'dash' (either '-' or '+') character that is before an
   '@' character. */

while(index && index < dline + BUFMAX && *index != '@' && *index > 32)
	{
	if(*index == DASH)
		lastdash = index;
	index++;
	}

/* The following is needed so only the actual dash is decrypted, only the 
   cookie after the dash */

if(lastdash != index)
	lastdash++;


/* Copy the cookie between the last dash and the '@' in to the string 
   'date' (named so because the original Kiwi only encrypted a date
   stamp) */

counter = 0;

while(dline && lastdash > dline && lastdash[counter] > 32 && 
      lastdash[counter] != '@' && (lastdash + counter) < BUFMAX + dline)
	{
	date[counter] = lastdash[counter];
	/* Make cookie case insensitive */
        if(date[counter] >= 'A' && date[counter] <= 'Z')
		date[counter] += 32;
	counter++;
	}		

date[counter] = 0;

now = time(0);

/* Check to see if this is a personal friend who knows our "secret password"
   email address.  We only do this if the password is actually defined */

#ifdef PASSWORD
if(!strncmp(date,PASSWORD,BUFMAX))
	return 1;
#endif

/* Otherwise, check the encrypted contents */

/* Parsedate does the actual decryption, and convets it in to a 
   simple {data format, data word} structure */

parsedate(date,msg);

/* Is the cookie a timestamped message with long timeout */
if(msg->data < now + 604800 && msg->data > now - (86400 * DAYS_LONG) &&
   msg->format == FORMAT_LONG_TIMEOUT)
	return 1;	

/* Maybe it is an IP address where email obtained */
if(msg->format == FORMAT_IP_BLOCK)
	return 1;

/* Perhaps a five-letter desginated permanent address */
if(msg->format == FORMAT_FIVE_LETTERS)
	return 1;

/* Or it could be a timestamped message with short timeout */
if(msg->data < now + 604800 && msg->data > now - (86400 * DAYS_SHORT) &&
   msg->format == FORMAT_SHORT_TIMEOUT)
	return 1;	

/* Timestamped message with mid-length timeout?? */
if(msg->data < now + 604800 && msg->data > now - (86400 * DAYS_MID) &&
   msg->format == FORMAT_MID_TIMEOUT)
#ifdef MID_PROCESS
	return 2;
#else /* MID_PROCESS */
	return 1;	
#endif

/* None of the above?  It is bad data or time out of range -- we discard 
   the mail */
	return 0;

}

main()
{
char line[BUFMAX][BUFMAX];
char dline[BUFMAX];
char addr[BUFMAX];

int stream[2]; /* used for piping */

char *deliv, *subj, *from, *to, *date;

int linenum,inheader,good, file, counter;

#ifdef BOUNCEMAIL
FILE *bouncefile;
int bytesread;
#endif

linenum = good = 0;

from = to = date = subj = deliv = 0;

inheader = 1;

#ifdef QMAIL
deliv = (char *)getenv("DTLINE");
#endif

/* We read the headers and look for headers like "To", "From", "Date", and
   so on.  While we don't use all of these headers right now, we may in
   future releases where we actually return undelivered messages */

while(!feof(stdin) && inheader && linenum < BUFMAX) 
	{
	fgets(line[linenum],(BUFMAX - 1),stdin);
#ifndef QMAIL
	/* On newer Sendmail systems, the actual address the message was
           delivered to is in a "Received:" header, on a line beginning 
	   `<tab>for' */
	if(!strncasecmp(line[linenum],"\tfor",4))
		deliv = line[linenum];
#endif /* QMAIL */
	if(!strncasecmp(line[linenum],"From:",5))
		from = line[linenum];
	if(!strncasecmp(line[linenum],"To:",3))
		to = line[linenum];
	if(!strncasecmp(line[linenum],"Date:",5))
		date = line[linenum];
	if(!strncasecmp(line[linenum],"Subject:",8))
		subj = line[linenum];
	if(linenum > 0 && line[linenum][0] == '\n' && 
           line[linenum - 1][0] == '\n')
		inheader = 0;
	linenum++;
	}

/* If there is a line containing the email that this was a "RCPT TO",
   we parse that line to see if we accept the message in question */
if(deliv && *deliv)
	good = goodtime(deliv);

/* Let us find out the address this mail was supposibly from */
*addr = 0;
if(from)
	getaddress(from,addr);


if(good == 1) /* If it was accepted and is *not* mid_timeout */
	{

/* If we are to append this to a mailbox file */
#ifdef APPEND
	/* To do: better lockfile handling, mail location handling */
	if((file = open(MYMAILBOX,
	            O_WRONLY|O_CREAT|O_APPEND,S_IRUSR|S_IWUSR)) == -1)
		{
		fprintf(stderr,"Fatal error: open failed\n");
		exit(44);
		}
	/* Lock the file while we do the append */
	flock(file, LOCK_EX);
	if(file != -1)
		{
#ifdef QMAIL
		/* We need to add a "From " line when using qmail with
		   a pipe */
		if(write(file,
	          "From nobody@localhost Mon Sep 14 00:00:00 1998\n",47) ==
                  -1)
			{
			fprintf(stderr,"Fatal error: write failed!\n");
			exit(54);
			}
		/* If there is a "Delivered-To" header, make it visible,
		   prepending it with an "X-" */
		if(deliv && *deliv)
			{
			if(write(file,"X-",2) == -1)
				{
				fprintf(stderr,"Fatal error: write failed!\n");
				exit(54);
				}
			if(write(file,deliv,strlen(deliv)) == -1)
				{
				fprintf(stderr,"Fatal error: write failed!\n");
				exit(54);
				}
			}
#endif /* QMAIL */
		for(counter = 0;counter < linenum ; counter++)
			if(write(file,line[counter],strlen(line[counter]))==-1)
				{
				fprintf(stderr,"Fatal error: write failed!\n");
				exit(54);
				}
		while(!feof(stdin))
			{
			fgets(line[0],(BUFMAX - 1),stdin);
			if(!feof(stdin))
				if(write(file,line[0],strlen(line[0])) == -1)
					{
					fprintf(stderr,"Write failed!\n");
					exit(54);
					}
			}
		}	
	flock(file, LOCK_UN);
	close(file);
#endif /* APPEND */
#ifdef FORWARD
	if(pipe(stream) != 0) /* Create a pipe */
		exit(-1);

	if(fork())
		{ /* parent */
		close(stream[1]); /* close the output stream */
        	dup2(stream[0],0); /* duplicate the standard input */
        	execl(SENDMAIL,"sendmail",MYADDRESS,(char *)0);
		}
	else
		{ /* child */
		close(stream[0]); /* close the input stream */
#ifdef QMAIL
		if(deliv && *deliv)
			{
			if(write(stream[1],"X-",2) == -1)
				{
				fprintf(stderr,"Write failed!\n");
				exit(54);
				}
			if(write(stream[1],deliv,strlen(deliv)) == -1)
				{
				fprintf(stderr,"Write failed!\n");
				exit(54);
				}
			}
#endif /* QMAIL */
		for(counter = 0;counter < linenum ; counter++)
			if(write(stream[1],line[counter],
			   strlen(line[counter])) == -1)
			   {
			   fprintf(stderr,"Write failed!\n");
			   exit(54);
			   }
		while(!feof(stdin))
			{
			fgets(line[0],(BUFMAX - 1),stdin);
			if(!feof(stdin))
				if(write(stream[1],line[0],strlen(line[0])) 
				    == -1)
				    {
				    fprintf(stderr,"Sendmail Write failed!\n");
				    exit(54);
				    }
			}
		}
#endif /* FORWARD */
	}
#ifdef MID_PROCESS
else if(good == 2)
	{
	if(pipe(stream) != 0)
		exit(-1);

	if(fork())
		{ /* parent */
		close(stream[1]); /* close the output stream */
        	dup2(stream[0],0); /* duplicate the standard input */
        	execl(MID_PROCESS,MID_PROCESS,MYADDRESS,(char *)0);
		}
	else
		{ /* child */
		close(stream[0]); /* close the input stream */
#ifdef QMAIL
		if(deliv && *deliv)
			{
			if(write(stream[1],"X-",2) == -1)
				{
				fprintf(stderr,"Write failed!\n");
				exit(54);
				}
			if(write(stream[1],deliv,strlen(deliv)) == -1)
				{
				fprintf(stderr,"Write failed!\n");
				exit(54);
				}
			}
#endif /* QMAIL */
		for(counter = 0;counter < linenum ; counter++)
			write(stream[1],line[counter],strlen(line[counter]));
		while(!feof(stdin))
			{
			fgets(line[0],(BUFMAX - 1),stdin);
			if(!feof(stdin))
				if(write(stream[1],line[0],strlen(line[0]))
				   == -1)
					{
					fprintf(stderr,"Write failed!\n");
					exit(54);
					}
			}
		}
	}
#endif /* MID_PROCESS */
else /* invalid Kiwi data  */
	{

#ifdef BOUNCEMAIL
	/* Bounce the BOUNCEMAIL to the person who sent us the message */
	if(addr && *addr) 
		{
		if(pipe(stream) != 0)
			exit(-1);

		if(fork())
			{ /* parent */
			close(stream[1]); /* close the output stream */
        		dup2(stream[0],0); /* duplicate the standard input */
        		execl(SENDMAIL,"sendmail",addr,(char *)0);
			}
		else
			{ /* child */
                	/* Send a "mail rejected" letter */
			close(stream[0]); /* close the input stream */

			/* Read in the bounce message, if defined */
			/* We open up BOUNCEFILE, read from it, and send it to
	       	            them as a bounce */
			if((bouncefile = fopen(BOUNCEMAIL,"r")) == NULL)
				{
				fprintf(stderr,"Fopen of %s failed!\n",
                                        BOUNCEMAIL);
				exit(54);
				}
			while(!feof(bouncefile))
				{
				fgets(line[0],(BUFMAX - 1),bouncefile);
				if(write(stream[1],line[0],
                                   strlen(line[0])) == -1)
					{
					fprintf(stderr,"Write failed!\n");	
					exit(54);
					}
				}
			fclose(bouncefile);

			/* Now, send them a copy of the rejected mail */
			/* Headers of rejected mail */
			for(counter = 1;counter < linenum ; counter++)
				{
				if(write(stream[1],"> ",2) == -1)
					{
					fprintf(stderr,"Write failed!\n");
					exit(54);	
					}
				if(write(stream[1],line[counter],
                        	   strlen(line[counter])) == -1)
					{
					fprintf(stderr,"Write failed!\n");
					exit(54);
					}
				}
			/* Body of rejected mail */
			while(!feof(stdin))
				{
				fgets(line[0],(BUFMAX - 1),stdin);
				if(!feof(stdin))
					if(write(stream[1],line[0],
             				         strlen(line[0])) == -1)
						{
						fprintf(stderr,
          					        "Write failed!\n");
						exit(54);
						}
				}
			}
		}

#endif /* BOUNCEMAIL */

	/* We make a log of the reject message, and put the
           message in the bit bucket */
	file = open(MAILLOG,O_WRONLY|O_CREAT|O_APPEND,
	            S_IRUSR|S_IWUSR);
	flock(file, LOCK_EX);
	if(file != -1)
		{
		write(file,"Message from ",13);
		if(addr && *addr)
			write(file,addr,strlen(addr));
		else
			write(file,"(address not defined)",21);
		write(file," sent to ",9);
		if(deliv)
        		getaddress(deliv,addr);
		if(addr && *addr)
			write(file,addr,strlen(addr));
		else
			write(file,"(address not defined)",21);
		write(file," rejected.\n",11);
		close(file);
		}
	flock(file, LOCK_UN);
	close(file);

	/* This is needed to avoid a broken pipe, which procmail and other
           mail delivery agents do not like */
	while(!feof(stdin))
		{
		fgets(line[0],(BUFMAX - 1),stdin);
		}

	}

exit(0); /* Needed or you will confuse your MTA */

}

