/* Copyright 1998 Sam Trenholme
   This code is released to the public domain */

/* For a given Blowfish-32 key, spit out the sboxes and pboxes */

#include <sys/types.h>
#include <stdio.h>
#include "kiwi.h"

u_int16_t sbox[4][16] = { { 0xd131, 0x98df, 0x2ffd, 0xd01a,
                            0xb8e1, 0x6a26, 0xba7c, 0xf12c,
                            0x24a1, 0xb391, 0x0801, 0x858e,
                            0x6369, 0x7157, 0xa458, 0xf493,
                       }, { 0x4b7a, 0xb5b3, 0xdb75, 0xc419,
                            0xad6e, 0x49a7, 0x9cee, 0x8fed,
                            0xecaa, 0x699a, 0x5664, 0xc2b1,
                            0x1936, 0x7509, 0xa059, 0xe418,
                       }, { 0xe93d, 0x9481, 0xf64c, 0x9469,
                            0x4115, 0x7602, 0xbcf4, 0xd4a2,
                            0xd408, 0x3320, 0x43b7, 0x5000,
                            0x1e39, 0x9724, 0x1421, 0xbf8b,
                       }, { 0x3a39, 0xd3fa, 0xabc2, 0x5ac5,
                            0x5cb0, 0x4fa3, 0xd382, 0x99bc,
                            0xd511, 0xbf0f, 0xd62d, 0xc700,
                            0xb78c, 0x21a1, 0xb26e, 0x6a36 } };

u_int16_t pbox[18] = { 0x243f, 0x85a3, 0x1319, 0x0370,
                       0xa409, 0x299f, 0x082e, 0xec4e,
                       0x4528, 0x38d0, 0xbe54, 0x34e9,
                       0xc0ac, 0xc97c, 0x3f84, 0xb547,
                       0x9216, 0x8979 };

#define f(x) ( ( ( sbox[0][(x & 0xF000) >> 12] + sbox[1][(x & 0x0F00) >> 8] ) \
             ^ sbox[2][(x & 0x00F0) >> 4] ) + sbox[3][x & 0x000F] )

u_int16_t l=0x0000,r=0x0000;

void key_sched( u_int16_t *pl, u_int16_t *pr ) 
{
   
l ^= pbox[0];
r ^= f(l) ^ pbox[1];
l ^= f(r) ^ pbox[2];
r ^= f(l) ^ pbox[3];
l ^= f(r) ^ pbox[4];
r ^= f(l) ^ pbox[5];
l ^= f(r) ^ pbox[6];
r ^= f(l) ^ pbox[7];
l ^= f(r) ^ pbox[8];
r ^= f(l) ^ pbox[9];
l ^= f(r) ^ pbox[10];
r ^= f(l) ^ pbox[11];
l ^= f(r) ^ pbox[12];
r ^= f(l) ^ pbox[13];
l ^= f(r) ^ pbox[14];
r ^= f(l) ^ pbox[15];
l ^= f(r) ^ pbox[16];
r ^= pbox[17];

*pr = l;
*pl = r;

l = r;
r = *pr;

}

main()
{

char key[] = KEY;

int counter, index;
char *point;

/* I really ought to allow ASCII nulls in the key */

if(!&key[0] || !key[0])
	{
	fprintf(stderr,"Need valid KEY data\n");
	exit(44);
	}

point = key;

index = 0;

for(counter=0;counter<18;counter++)
	{
	u_int16_t temp;
	temp = (*point & 0xff) << 8;
	point++;
	index++;
	if(!*point || index >= 28)
		{
		point = key;
		index = 0;
		}
	temp |= *point & 0xff;
	pbox[counter] ^= temp;
	point++;
	index++;
	if(!*point || index >= 28)
		{
		point = key;
		index = 0;
		}
	}

for(counter=0;counter<18;counter+=2)
	key_sched(&pbox[counter],&pbox[(counter + 1)]);

for(counter=0;counter<4;counter++)
	for(index=0;index<16;index+=2)
		key_sched(&sbox[counter][index],&sbox[counter][(index + 1)]);

printf("/* Copyright 1998 Sam Trenholme
   This program is released to the public domain. */

/* This program is automaticlly generated by make_scramble.c */

#include <sys/types.h>\n\n");

for(counter=0;counter<4;counter++)
	{
	printf("u_int16_t sbox%d[16] = { ",counter + 1);
	for(index=0;index<16;index++)
		{
		printf("0x%04x",sbox[counter][index]);
		if(index % 4 == 3 && index != 15)
			printf(", \n");
		else if(index != 15)
			printf(", ");
		else
			printf("\n");
		}
	printf("}; \n\n");
	}

printf(
"#define f(x) ( ( ( sbox1[(x & 0xF000) >> 12] + sbox2[(x & 0x0F00) >> 8] ) \
                 ^ sbox3[(x & 0x00F0) >> 4] ) + sbox4[x & 0x000F] )

u_int32_t scramble( u_int32_t plaintext ) 
{
u_int16_t l,r;
   
r = plaintext & 0xFFFF;
l = (plaintext >> 16) & 0xFFFF;

l ^= 0x%04x;\n",pbox[0]);

for(counter=1;counter<17;counter++)
	{
	if(counter % 2)	
		printf("r ^= f(l) ^ 0x%04x;\n",pbox[counter]);
	else	
		printf("l ^= f(r) ^ 0x%04x;\n",pbox[counter]);
	}

printf("r ^= 0x%04x;

return ((r << 16) | (l & 0xFFFF));

}

u_int32_t unscramble( u_int32_t plaintext ) 
{
u_int16_t l,r;
   
l = plaintext & 0xFFFF;
r = (plaintext >> 16) & 0xFFFF;

r ^= 0x%04x;\n",pbox[17],pbox[17]);

for(counter=16;counter>0;counter--)
        {
	if(counter % 2)
		printf("r ^= f(l) ^ 0x%04x;\n",pbox[counter]);
	else
		printf("l ^= f(r) ^ 0x%04x;\n",pbox[counter]);
	}

printf("l ^= 0x%04x;

return ((l << 16) | (r & 0xFFFF));

}\n",pbox[0]);

exit(0);

}

