/* Copyright 1998 Sam Trenholme
   This code is released to the public domain. */

#include <sys/types.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <fcntl.h>
#include "kiwi.h"

/* convert an encrypted cookie like jlsz63a into the data it contains */

void *parsedate(char *datestring, message *msg)
{

	char b32map[] = MAPPING;

        int counter, index;

        u_int32_t shift, temp;
	u_int16_t data[2];
	u_int8_t parity, paritycheck;

	msg->format = FORMAT_INVALID;
	msg->data = 0;

	/* If we do not have a string to look at, exit abnormally */
	if(!datestring || !*datestring)
		return;

	/* Go through the 7-character cookie, and convert it to binary
	   data */
        for(counter=6;counter>=0;counter--) 
     		{
		/* We shift left if we are not on the right most end */
                if(counter<6)
                	shift <<= 5;

		/* What we do is go through the entire 32-character possible
		   mappings (where 0 is 'a', 1 is 'b', and so on), and
		   see if the character we are looking at has a mapping */

		/* For all 32 possible mappings... */
                for(index=0;index<32;index++) 
                	{
			/* If the character we are looking at matches
			   the mapping we are looking at... */
			if(datestring[counter] == b32map[index])
				/* Then add the information to the binary
				   data */
				if(counter<6)
					shift |= index;
				else	
					/* If we are looking at the right
					   most character, we have two
					   pieces of binary data--the two
					   bits of data, and three bits
					   of parity */
					{
					shift = index & 0x03;
					parity = index & 0x1C;
					parity >>= 2;
					}
			}
                }

	/* Since the data (in shift) is cipher text, we use the unscramble
	   routine to make it plaintext */
	shift = unscramble(shift);

	/* We copy the plaintext over to a temporary variable, in preparation
	   for making sure the parity is correct */
	temp = shift;

	/* We look at the plaintext and determine what the correct parity
	   for it should be */	
	paritycheck = 0;
	for(counter = 0; counter < 11; counter++)
                {
                paritycheck ^= temp & 0x07;
                temp >>= 3;
                }

	/* If the parity is incorrect, we return the format as "BAD_PARITY" */
	if(paritycheck != parity)
		{
		msg->format = FORMAT_BAD_PARITY;
		msg->data = 0;
		return;
		}

	/* If the high four bits are "1010" (A), we know it is a time 
	   stamped message with a long timeout */
	if((shift & 0xF0000000) == 0xA0000000)
		{
		shift <<= 4;
		msg->format = FORMAT_LONG_TIMEOUT;
		}

	/* If the high four bits are "1000" (8), we know it is a time
	   stamped message with a short timeout */
	else if((shift & 0xF0000000) == 0x80000000)
		{
		shift <<= 4;
		msg->format = FORMAT_SHORT_TIMEOUT;
		}
 
	/* If the high four bits are "1001" (9), we know it is a time
	   stamped message with a medium timeout */
	else if((shift & 0xF0000000) == 0x90000000)
		{
		shift <<= 4;
		msg->format = FORMAT_MID_TIMEOUT;
		}

	/* If the high four bits are "1100" (C), we know the message is
	   an IP, within a 16-ip range */	
	else if((shift & 0xF0000000) == 0xC0000000)
		{
		shift <<= 4;
		msg->format = FORMAT_IP_BLOCK;
		}

	/* If the high 12 bits are 0x7FF, we know the message is a
	   timestamp that changes value every hour or so (used for moving
	   a directory or some other information once an hour, mail
	   sent to a cookie in this form is marked as "bad" and discarded) */
	else if((shift & 0xFFF00000) == 0x7FF00000)
		{
		shift <<= 12;
		msg->format = FORMAT_HIDDEN_DIR;
		}

	/* If the high 7 bits are "0111110" (7C), the remaining 25 bits are
           a five-letter message (where each letter uses five bits) */
	else if((shift & 0xFE000000) == 0x7C000000)
		{
		msg->format = FORMAT_FIVE_LETTERS;
		}

	/* We can't grok the message, so we consider it an invalid message, 
   	   discard it if it is a mail message */
	else 
		{
		msg->format = FORMAT_INVALID;
		shift = 0;
		}

	msg->data = shift;	

	return;

}

