/* Copyright 1998 Sam Trenholme
   This code is released to the public domain. */

/* This is a sendmail wrapper which is used to replace the email address by 
   a cryptographic string */

#include <unistd.h>
#include <time.h>
#include <stdio.h>
#include <sys/types.h>
#include "kiwi.h"

int 
main(int argc,char **argv)
{

/* Pretty crude, we take anything that looks like an email address and put
   a +crypto after it, with the execption of "Message-ID" headers */

char *replyto; 

char line[BUFMAX];
char answer[BUFMAX];
char crypto[8];

int reply=0; /* was there a reply-to line already? */

int counter, stream[2];

int inheader = 1, blankline = 0, addrline = 0;

u_int32_t data;

int stat; /* Used by the wait() system call so we don't get a zombie */

/* Set up the crypto with the encrypted time stamp */

data = 0xA0000000; /* 1010 indicates time stamp right shifted 4 bits */
data |= (time(0) & 0xFFFFFFFF) >> 4; 
/* Not year 2038 complient, easy to adapt */

/* Do the actual encryption */
cryptdate(data,crypto);

/* the string crypto now has the encrypted time stamp */

/* prepare the pipe.  To do: add error correction */

if(pipe(stream) != 0)
	exit(-1);

if(fork())
	{ /* parent */
	close(stream[1]); /* close the output stream */
	dup2(stream[0],0); /* duplicate the standard input */
	execv(SENDMAIL,argv); /* Run sendmail */
	/* If we get here, there is a problem */
	fprintf(stderr,"Fatal: Can not start sendmail!\n");
	exit(1);
	}
else
	{ /* child */
  	close(stream[0]); /* close the input stream */
	dup2(stream[1],1); /* duplicate the standard output to the pipe.
			      In other words, the standard output now goes
			      in to the pipe (sendmail) instead of to the 
			      tty of the process */
	*line = 0; /* Initialize the line for processing */
	/* Go through the message to be processed, processing lines as
	   needed */
        while(!feof(stdin))
		{

		/* If this line is a blank line with just a linefeed on
		   it, we set a flag saying "We are no longer in the header */
		/* should I allow any number of spaces in a blank line
                   be a legitimate separator? */
		if((*line == '\n' || *line == '\r'))
			inheader=0;

		/* If the line starts with a letter, it is not a 
		   continuation of the previous header line */
		if((*line >= 'A' && *line <= 'Z' ) || 
		   (*line >= 'a' && *line <= 'z' ))
			addrline=0;

		/* If the header line we are looking at is not a "Message-ID",
		   "To", "Cc", or similar header, and we are in the headers, 
		   add the cookie (and a "+"/"-" symbol) before any '@' on 
		   the line */
		if(strncasecmp(line,"Message-ID:",11) 
                   && strncasecmp(line,"To:",3) 
                   && strncasecmp(line,"Cc:",3) 
		   && strncasecmp(line,"Bcc:",4)
		   && strncasecmp(line,"In-Reply-To:",12)
		   && strncasecmp(line,"Content-",8)
		   && strncasecmp(line,"Subject:",8)
                   && inheader && !addrline)
			{
			counter = 0;
			/* We are using the 'replyto' variable because it
			   saves us the bother of initializing another
			   variable */
			replyto = line;

			/* If the line is a "X-Sender:" header, we do not
			   make the line visible to the sendmail program.
			   (Pine has an annoying habit of putting in 
			    that header).  Otherwise, while the line does 
			   not have the string terminate, and we are not
			   going over the bounds of the string, we read
			   and modify the string in question */
			while(counter < BUFMAX && *replyto && 
		              strncasecmp(line,"X-Sender:",9))
				{
				/* If the character in question is an '@',
				   we prepend it with a cookie */
				if(*replyto == '@')
					{
					putc(DASH,stdout);
					printf("%s",crypto);
					}
				/* We print out the character we are
				   looking at */
				putc(*replyto,stdout);
				replyto++;
				counter++;
				}
			}
		else
			{
			/* If we are out of the header, or are a "To",
			   "Cc", or "Message-ID" header, we just output 
			   the line as is */
			printf("%s",line);	
			/* Making a note if it is an addrline 
			   In other words, if it is a line we do not
			   Kiwi-mangle */
			if(!strncasecmp(line,"Message-ID:",11)
			   || !strncasecmp(line,"To:",3)
			   || !strncasecmp(line,"Cc:",3)
			   || !strncasecmp(line,"Bcc:",4)
		   	   || !strncasecmp(line,"Content-",8)
			   || !strncasecmp(line,"In-Reply-To:",12))
				addrline = 1;
			}

		fgets(line,(BUFMAX - 1),stdin);	
		}	
	/* No zombies, if we can avoid it */
	wait(&stat);
	}

exit(0); /* this is important, otherwise our mail user agent thinks that'
            the message was not successfully delivered */

}

